#
# == Class: mcoconfig::active
#
# Configures an ActiveMQ instance that MCollective will use as a broker.
#
# === Parameters
#
# [*activemq_cn*]
#   The CN of the Puppet certificate to be used by ActiveMQ.
#
# [*mcollective_password*]
#   The password for the mcollective user.
#
# [*admin_password*]
#   The password for the admin user.
#
# [*keystore_password*]
#   The password for the ActiveMQ truststore and keystore.
#
# [*activemq_user*]
#   The username that ActiveMQ runs as.
#
# [*activemq_config_dir*]
#   The path to the ActiveMQ configuration directory.
#
# [*activemq_etc_dir*]
#   The location of the ActiveMQ keystore, truststore and instances.
#
# [*activemq_data_dir*]
#   The location of the ActiveMQ database.
#
# [*puppet_ca_dir*]
#   The location of the Puppet CA.
#
# [*os_release*]
#   The OS release to pass into the package manager.
#
# [*java_home*]
#   The path for the JAVA_HOME environment variable and options file.
#
#
# === Examples
#
#  class { "mcoconfig::activemq":
#  }
#
# === Authors
#
# David McNicol <david@mcnicks.org>
#
# === Copyright
#
# Copyright 2014 David McNicol
#

class mcoconfig::activemq (
  $activemq_cn,
  $mcollective_password,
  $admin_password,
  $keystore_password,
  $activemq_user = $mcoconfig::activemq::params::activemq_user,
  $activemq_config_dir = $mcoconfig::activemq::params::activemq_config_dir,
  $activemq_etc_dir = $mcoconfig::activemq::params::activemq_etc_dir,
  $activemq_data_dir = $mcoconfig::activemq::params::activemq_data_dir,
  $puppet_ca_dir = $mcoconfig::activemq::params::puppet_ca_dir,
  $os_release = $mcoconfig::activemq::params::os_release,
  $java_home = $mcoconfig::activemq::params::java_home
) inherits mcoconfig::activemq::params {

  # Install the ActiveMQ package.

  class { 'mcoconfig::activemq::install':
    os_release => $os_release
  } ->

  # Create the required configuration.

  class { 'mcoconfig::activemq::config':
    activemq_user        => $activemq_user,
    activemq_config_dir  => $activemq_config_dir,
    activemq_etc_dir     => $activemq_etc_dir,
    activemq_data_dir    => $activemq_data_dir,
    activemq_ca_cert     => "${puppet_ca_dir}/certs/ca.pem",
    activemq_cert        => "${puppet_ca_dir}/certs/${activemq_cn}.pem",
    activemq_key         => "${puppet_ca_dir}/private_keys/${activemq_cn}.pem",
    mcollective_password => $mcollective_password,
    admin_password       => $admin_password,
    keystore_password    => $keystore_password,
    java_home            => $java_home,
    notify               => Service['activemq']
  }

  # Start the ActiveMQ service.

  service { 'activemq':
    ensure => 'running',
    enable => true
  }
}
