#
# == Class: mcoconfig::activemq::config
#
# Creates keystores and config files for ActiveMQ.
#
# === Parameters
#
# [*activemq_user*]
#   The username that ActiveMQ runs as.
#
# [*activemq_config_dir*]
#   The path to the ActiveMQ configuration directory.
#
# [*activemq_etc_dir*]
#   The location of the ActiveMQ keystore, truststore and instances.
#
# [*activemq_data_dir*]
#   The location of the ActiveMQ runtime data.
#
# [*activemq_ca_cert*]
#   The CA certificate that ActiveMQ should use.
#
# [*activemq_cert*]
#   The certificate that ActiveMQ should use.
#
# [*activemq_key*]
#   The private key that ActiveMQ should use.
#
# [*mcollective_password*]
#   The password for the mcollective user.
#
# [*admin_password*]
#   The password for the admin user.
#
# [*keystore_password*]
#   The password to use for the truststore and keystore.
#
# [*java_home*]
#   The path for the JAVA_HOME environment variable and options file.
#
# === Authors
#
# David McNicol <david@mcnicks.org>
#
# === Copyright
#
# Copyright 2014 David McNicol
#

class mcoconfig::activemq::config (
  $activemq_user,
  $activemq_config_dir,
  $activemq_etc_dir,
  $activemq_data_dir,
  $activemq_ca_cert,
  $activemq_cert,
  $activemq_key,
  $mcollective_password,
  $admin_password,
  $keystore_password,
  $java_home
) {

  # Create the ActiveMQ options file.

  file { "${activemq_config_dir}/options":
    ensure  => 'present',
    content => template('mcoconfig/options.erb'),
    owner   => $activemq_user,
    group   => $activemq_user,
    mode    => '0600'
  }
  
  # Create the ActiveMQ XML file.

  file { "${activemq_config_dir}/activemq.xml":
    ensure  => 'present',
    content => template('mcoconfig/activemq.xml.erb'),
    owner   => $activemq_user,
    group   => $activemq_user,
    mode    => '0600'
  }

  # Create the truststore containing the CA cert.

  java_ks { 'activemq.ts':
    ensure       => 'latest',
    certificate  => $activemq_ca_cert,
    target       => "${activemq_etc_dir}/activemq.ts",
    trustcacerts => true,
    password     => $keystore_password
  } ->

  # Fix permissions on the truststore.

  file { "${activemq_etc_dir}/activemq.ts":
    owner => $activemq_user,
    group => $activemq_user,
    mode  => '0600'
  }

  # Create the keystore containing the activemq cert and private key.

  java_ks { 'activemq.ks':
    ensure      => 'latest',
    certificate => $activemq_cert,
    private_key => $activemq_key,
    target      => "${activemq_etc_dir}/activemq.ks",
    password    => $keystore_password
  } ->

  # Fix permissions on the keystore.

  file { "${activemq_etc_dir}/activemq.ks":
    owner => $activemq_user,
    group => $activemq_user,
    mode  => '0600'
  }

  # Create link to make the default ActiveMQ instance active on Debian.

  if $::osfamily =~ /(Debian|Ubuntu)/ {
    file { "${activemq_etc_dir}/instances-enabled/main":
      ensure => 'link',
      target => "${activemq_etc_dir}/instances-available/main"
    }
  }
}
