#
# == Class: mcoconfig::addclient
#
# Defined type that creates MCollective client data on the Puppet master.
#
# === Parameters
#
# [*client_fqdn*]
#   The FQDN of the client.
#
# [*username*]
#   The username to set up as the client.
#
# === Examples
#
#  Single client user:
#
#    mcoclient::addclient { "client.domain.com":
#        username => "user"
#    }
#
#  Multiple client users on same client:
#
#    mcoclient::addclient { "user1-client.domain.com":
#        client_fqdn => "client.domain.com",
#        username => "user1"
#    }
#
#    mcoclient::addclient { "user2-client.domain.com":
#        client_fqdn => "client.domain.com",
#        username => "user2"
#    }
#
# === Authors
#
# David McNicol <david@mcnicks.org>
#
# === Copyright
#
# Copyright 2014 David McNicol
#

define mcoconfig::addclient (
  $username,
  $client_fqdn = $title
) {

  # Set up the file store.

  include 'mcoconfig::filestore'

  $filestore_root = $mcoconfig::filestore::filestore_root
  $filestore_clients = $mcoconfig::filestore::filestore_clients
  $filestore_private = $mcoconfig::filestore::filestore_private
  $puppet_ca_dir = $mcoconfig::filestore::puppet_ca_dir

  $client_dir = "${filestore_root}/${filestore_clients}"
  $private_dir = "${filestore_root}/${filestore_private}/${client_fqdn}"

  # Ensure that a directory exists in the private file store for the client.

  file { $private_dir:
    ensure  => 'directory',
    owner   => 'puppet',
    group   => 'puppet',
    mode    => '0640',
    require => File["${filestore_root}/${filestore_private}"]
  }

  # Create the client certificate and private key.

  $client_cert_name = "${username}_${client_fqdn}"

  exec { $client_cert_name:
    path    => [ '/bin/', '/sbin/' , '/usr/bin/', '/usr/sbin/' ],
    command => "puppet cert generate ${client_cert_name}",
    creates => "${puppet_ca_dir}/private_keys/${client_cert_name}.pem"
  }

  # Copy the client cert and key into the filestore.

  file { "${client_dir}/${client_cert_name}.pem":
    ensure  => 'present',
    owner   => 'puppet',
    group   => 'puppet',
    mode    => '0644',
    source  => "${puppet_ca_dir}/certs/${client_cert_name}.pem",
    require => [
      File[$client_dir],
      Exec[$client_cert_name]
    ]
  }

  file { "${private_dir}/${client_cert_name}.pem":
    ensure  => 'present',
    owner   => 'puppet',
    group   => 'puppet',
    mode    => '0640',
    source  => "${puppet_ca_dir}/private_keys/${client_cert_name}.pem",
    require => [
      File[$private_dir],
      Exec[$client_cert_name]
    ]
  }
}
