#
# == Class: mcoconfig::filestore
#
# Creates the the MCollective file store.
#
# === Parameters
#
# [*filestore_root*]
#   The path to the MCollective filestore root directory.
#
# [*filestore_shared*]
#   The subdirectory under the filestore root that holds shared certs.
#
# [*filestore_clients*]
#   The subdirectory under the filestore root that holds client certs.
#
# [*filestore_private*]
#   The subdirectory under the filestore root that holds client private keys.
#
# [*puppet_ca_dir*]
#   The location of the Puppet CA.
#
# [*puppet_service_name*]
#   The name of the Puppet service. Used for notifying Puppet to restart.
#
# === Examples
#
#  include mcoconfig::filestore
#
# === Authors
#
# David McNicol <david@mcnicks.org>
#
# === Copyright
#
# Copyright 2014 David McNicol
#

class mcoconfig::filestore (
  $filestore_root = $mcoconfig::params::filestore_root,
  $filestore_shared = $mcoconfig::params::filestore_shared,
  $filestore_clients = $mcoconfig::params::filestore_clients,
  $filestore_private = $mcoconfig::params::filestore_private,
  $puppet_ca_dir = $mcoconfig::params::puppet_ca_dir,
  $puppet_service_name = $mcoconfig::params::puppet_service_name
) inherits mcoconfig::params {

  # Create the directory structure for the filestore.

  file { $filestore_root:
    ensure => 'directory',
    owner  => 'puppet',
    group  => 'puppet',
    mode   => '0640'
  }

  file { "${filestore_root}/${filestore_shared}":
    ensure  => 'directory',
    owner   => 'puppet',
    group   => 'puppet',
    mode    => '0640',
    require => File[$filestore_root]
  }

  file { "${filestore_root}/${filestore_clients}":
    ensure  => 'directory',
    owner   => 'puppet',
    group   => 'puppet',
    mode    => '0640',
    require => File[$filestore_root]
  }

  file { "${filestore_root}/${filestore_private}":
    ensure  => 'directory',
    owner   => 'puppet',
    group   => 'puppet',
    mode    => '0640',
    require => File[$filestore_root]
  }

  service { $puppet_service_name:
    ensure    => 'running',
    subscribe => [
      File["${filestore_root}/${filestore_shared}"],
      File["${filestore_root}/${filestore_clients}"],
      File["${filestore_root}/${filestore_private}"]
    ]
  }

  # Copy the Puppet CA certificate into the shared filestore.

  file { 'ca.pem':
    ensure => 'present',
    path   => "${filestore_root}/${filestore_shared}/ca.pem",
    owner  => 'puppet',
    group  => 'puppet',
    mode   => '0644',
    source => "${puppet_ca_dir}/certs/ca.pem"
  }

  # Create the shared server certificate and private key.

  exec { 'shared_server_cert':
    path    => [ '/bin/', '/sbin/' , '/usr/bin/', '/usr/sbin/' ],
    command => 'puppet cert generate mco_servers',
    creates => "${puppet_ca_dir}/private_keys/mco_servers.pem"
  }

  # Copy the shared server cert and key into the shared filestore.

  file { 'server_cert.pem':
    ensure  => 'present',
    path    => "${filestore_root}/${filestore_shared}/server_cert.pem",
    owner   => 'puppet',
    group   => 'puppet',
    mode    => '0644',
    source  => "${puppet_ca_dir}/certs/mco_servers.pem",
    require => Exec['shared_server_cert']
  }

  file { 'server_key.pem':
    ensure  => 'present',
    path    => "${filestore_root}/${filestore_shared}/server_key.pem",
    owner   => 'puppet',
    group   => 'puppet',
    mode    => '0640',
    source  => "${puppet_ca_dir}/private_keys/mco_servers.pem",
    require => Exec['shared_server_cert']
  }
}
