#
# == Class: mcoconfig::puppetmaster
#
# Used to configure the Puppet master as the middleware server running
# ActiveMQ and an MCollective server. Also configures the root user as a
# client.
#
# === Parameters
#
# [*mcollective_password*]
#   The password for the ActiveMQ mcollective user.
#
# [*admin_password*]
#   The password for the ActiveMQ admin user.
#
# [*keystore_password*]
#   The password for the ActiveMQ truststore and keystore.
#
# [*mcollective_plugins*]
#   An array of MCollective plugins to enable.
#
# === Examples
#
#  class { "mcoconfig::puppetmaster":
#      middleware_node => "puppet.domain.com",
#      mcollective_password => "NNNNNNNN",
#      admin_password => "MMMMMMMMM",
#      keystore_password => "OOOOOOOO"
#  }
#
# === Authors
#
# David McNicol <david@mcnicks.org>
#
# === Copyright
#
# Copyright 2014 David McNicol
#

class mcoconfig::puppetmaster (
  $mcollective_password,
  $admin_password,
  $keystore_password,
  $mcollective_plugins
) {

  # Configure the Puppet file store.

  include 'mcoconfig::filestore'

  # Configure ActiveMQ.

  class { 'mcoconfig::activemq':
    activemq_cn          => $::fqdn,
    mcollective_password => $mcollective_password,
    admin_password       => $admin_password,
    keystore_password    => $keystore_password
  }

  # Configure the MCollective server to connect it to the ActiveMQ server.

  class { 'mcoconfig':
    middleware_node      => $::fqdn,
    mcollective_password => $mcollective_password,
    mcollective_plugins  => $mcollective_plugins,
    require              => Class['mcoconfig::filestore']
  }

  # Create a new client for the root user on this node.

  mcoconfig::addclient { $::fqdn:
    username => 'root'
  } ->

  # Configure MCollective for the root user.

  mcoconfig::user { 'root':
    homedir => '/root'
  }
}
